@@ -1,7 +1,7 @@
 -- Prosody IM
--- Copyright (C) 2008-2009 Matthew Wild
--- Copyright (C) 2008-2009 Waqas Hussain
--- 
+-- Copyright (C) 2008-2010 Matthew Wild
+-- Copyright (C) 2008-2010 Waqas Hussain
+--
 -- This project is MIT/X11 licensed. Please see the
 -- COPYING file in the source package for more information.
 --
@@ -14,7 +14,8 @@ local tostring = tostring;
 local getstyle, getstring = require "util.termcolours".getstyle, require "util.termcolours".getstring;
 local do_pretty_printing = not os.getenv("WINDIR");
 
-module "logger"
+local _ENV = nil
+local _M = {}
 
 local logstyles = {};
 
@@ -25,7 +26,7 @@ if do_pretty_printing then
 	logstyles["error"] = getstyle("bold", "red");
 end
 
-function init(name)
+function _M.init(name)
 	--name = nil; -- While this line is not commented, will automatically fill in file/line number info
 	return 	function (level, message, ...)
 				if level == "debug" or level == "info" then return; end
@@ -33,7 +34,7 @@ function init(name)
 					local inf = debug.getinfo(3, 'Snl');
 					level = level .. ","..tostring(inf.short_src):match("[^/]*$")..":"..inf.currentline;
 				end
-				if ... then 
+				if ... then
 					print(name, getstring(logstyles[level], level), format(message, ...));
 				else
 					print(name, getstring(logstyles[level], level), message);
